searchData={"items":[{"type":"task","title":"mix nimble_parsec.compile","doc":"Compiles a parser from a template.\n\n    $ mix nimble_parsec.compile template.ex.exs\n\nThis task is useful to generate parsers that have no runtime dependency\non NimbleParsec.","ref":"Mix.Tasks.NimbleParsec.Compile.html"},{"type":"task","title":"Examples - mix nimble_parsec.compile","doc":"Let's define a template file:\n\n    # lib/my_parser.ex.exs\n    defmodule MyParser do\n      @moduledoc false\n\n      # parsec:MyParser\n      import NimbleParsec\n\n      date =\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n\n      time =\n        integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> optional(string(\"Z\"))\n\n      defparsec :datetime, date |> ignore(string(\"T\")) |> concat(time)\n\n      # parsec:MyParser\n    end\n\nAfter running:\n\n    $ mix nimble_parsec.compile lib/my_parser.ex.exs\n\nThe following file will be generated:\n\n    # lib/my_parser.ex\n    defmodule MyParser do\n      @moduledoc false\n\n      def datetime(binary, opts \\\\ []) do\n        ...\n      end\n\n      defp datetime__0(...) do\n        ...\n      end\n\n      ...\n    end\n\nThe file will be automatically formatted if using Elixir v1.6+.","ref":"Mix.Tasks.NimbleParsec.Compile.html#module-examples"},{"type":"task","title":"Options - mix nimble_parsec.compile","doc":"* `-o` - configures the output location. Defaults to the input\n    file without its last extension","ref":"Mix.Tasks.NimbleParsec.Compile.html#module-options"},{"type":"module","title":"NimbleParsec","doc":"`NimbleParsec` is a simple and fast library for text-based parser\ncombinators.\n\nCombinators are composed programatically and compiled into multiple\nclauses with binary matching. This provides the following benefits:\n\n  * Performance: since it compiles to binary matching, it leverages\n    many Erlang VM optimizations to generate a fast parser code with\n    low memory usage\n\n  * Composable: this library does not rely on macros for building and\n    composing parsers, therefore they are fully composable. The only\n    macros are `defparsec/3` and `defparsecp/3` which emit the compiled\n    clauses with  binary matching\n\n  * No runtime dependency: after compilation, the generated parser\n    clauses have no runtime dependency on `NimbleParsec`. This opens up\n    the possibility to compile parsers and do not impose a dependency on\n    users of your library\n\n  * No footprints: `NimbleParsec` only needs to be imported in your modules.\n    There is no need for `use NimbleParsec`, leaving no footprints on your\n    modules\n\nThe goal of this library is to focus on a set of primitives for writing\nefficient parser combinators. The composition aspect means you should be\nable to use those primitives to implement higher level combinators.\n\nNote this library does not handle low-level binary parsing. In such cases,\nwe recommend using [Elixir's bitstring syntax](https://hexdocs.pm/elixir/Kernel.SpecialForms.html#%3C%3C%3E%3E/1).","ref":"NimbleParsec.html"},{"type":"module","title":"Examples - NimbleParsec","doc":"```elixir\ndefmodule MyParser do\n  import NimbleParsec\n\n  date =\n    integer(4)\n    |> ignore(string(\"-\"))\n    |> integer(2)\n    |> ignore(string(\"-\"))\n    |> integer(2)\n\n  time =\n    integer(2)\n    |> ignore(string(\":\"))\n    |> integer(2)\n    |> ignore(string(\":\"))\n    |> integer(2)\n    |> optional(string(\"Z\"))\n\n  defparsec :datetime, date |> ignore(string(\"T\")) |> concat(time), debug: true\nend\n\nMyParser.datetime(\"2010-04-17T14:12:34Z\")\n#=> {:ok, [2010, 4, 17, 14, 12, 34, \"Z\"], \"\", %{}, {1, 0}, 20}\n```\n\nIf you add `debug: true` to `defparsec/3`, it will print the generated\nclauses, which are shown below:\n\n```elixir\ndefp datetime__0( >,\n                 acc, stack, comb__context, comb__line, comb__column)\n     when x0 >= 48 and x0  = 48 and x1  = 48 and x2  = 48 and x3  = 48 and x4  = 48 and x5  = 48 and x6  = 48 and x7  = 48 and x8  = 48 and x9  = 48 and x10  = 48 and x11  = 48 and x12  = 48 and x13  >, acc, stack, comb__context, comb__line, comb__column) do\n  datetime__2(rest, [\"Z\"] ++ acc, stack, comb__context, comb__line, comb__column + 1)\nend\n\ndefp datetime__1(rest, acc, stack, context, line, column) do\n  datetime__2(rest, acc, stack, context, line, column)\nend\n\ndefp datetime__2(rest, acc, _stack, context, line, column) do\n  {:ok, acc, rest, context, line, column}\nend\n```\n\nAs you can see, it generates highly inlined code, comparable to\nhand-written parsers. This gives `NimbleParsec` an order of magnitude\nperformance gains compared to other parser combinators. Further performance\ncan be gained by giving the `inline: true` option to `defparsec/3`.","ref":"NimbleParsec.html#module-examples"},{"type":"function","title":"NimbleParsec.ascii_char/2","doc":"Defines a single ASCII codepoint in the given ranges.\n\n`ranges` is a list containing one of:\n\n  * a `min..max` range expressing supported codepoints\n  * a `codepoint` integer expressing a supported codepoint\n  * `{:not, min..max}` expressing not supported codepoints\n  * `{:not, codepoint}` expressing a not supported codepoint","ref":"NimbleParsec.html#ascii_char/2"},{"type":"function","title":"Examples - NimbleParsec.ascii_char/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :digit_and_lowercase,\n                empty()\n                |> ascii_char([?0..?9])\n                |> ascii_char([?a..?z])\n    end\n\n    MyParser.digit_and_lowercase(\"1a\")\n    #=> {:ok, [?1, ?a], \"\", %{}, {1, 0}, 2}\n\n    MyParser.digit_and_lowercase(\"a1\")\n    #=> {:error, \"expected ASCII character in the range '0' to '9', followed by ASCII character in the range 'a' to 'z'\", \"a1\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#ascii_char/2-examples"},{"type":"function","title":"NimbleParsec.ascii_string/3","doc":"Defines an ASCII string combinator with an exact length or `min` and `max`\nlength.\n\nThe `ranges` specify the allowed characters in the ASCII string.\nSee `ascii_char/2` for more information.\n\nIf you want a string of unknown size, use `ascii_string(ranges, min: 1)`.\nIf you want a literal string, use `string/2`.","ref":"NimbleParsec.html#ascii_string/3"},{"type":"function","title":"Examples - NimbleParsec.ascii_string/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :two_lowercase_letters, ascii_string([?a..?z], 2)\n    end\n\n    MyParser.two_lowercase_letters(\"abc\")\n    #=> {:ok, [\"ab\"], \"c\", %{}, {1, 0}, 2}","ref":"NimbleParsec.html#ascii_string/3-examples"},{"type":"function","title":"NimbleParsec.byte_offset/2","doc":"Puts the result of the given combinator as the first element\nof a tuple with the `byte_offset` as second element.\n\n`byte_offset` is a non-negative integer.","ref":"NimbleParsec.html#byte_offset/2"},{"type":"function","title":"NimbleParsec.choice/3","doc":"Chooses one of the given combinators.\n\nExpects at least two choices.","ref":"NimbleParsec.html#choice/3"},{"type":"function","title":"Beware! Char combinators - NimbleParsec.choice/3","doc":"Note both `utf8_char/2` and `ascii_char/2` allow multiple ranges to\nbe given. Therefore, instead this:\n\n    choice([\n      ascii_char([?a..?z]),\n      ascii_char([?A..?Z]),\n    ])\n\nOne should simply prefer:\n\n    ascii_char([?a..?z, ?A..?Z])\n\nAs the latter is compiled more efficiently by `NimbleParsec`.","ref":"NimbleParsec.html#choice/3-beware-char-combinators"},{"type":"function","title":"Beware! Always successful combinators - NimbleParsec.choice/3","doc":"If a combinator that always succeeds is given as a choice, that choice\nwill always succeed which may lead to unused function warnings since\nany further choice won't ever be attempted. For example, because `repeat/2`\nalways succeeds, the `string/2` combinator below it won't ever run:\n\n    choice([\n      repeat(ascii_char([?0..?9])),\n      string(\"OK\")\n    ])\n\nInstead of `repeat/2`, you may want to use `times/3` with the flags `:min`\nand `:max`.","ref":"NimbleParsec.html#choice/3-beware-always-successful-combinators"},{"type":"function","title":"Beware! Overlapping choices - NimbleParsec.choice/3","doc":"In case choices overlap, there is no guarantee which error will be the one\neffectively returned. For example, imagine this choice:\n\n    choice([\n      string(\" foo \"),\n      string(\" \")\n    ]\nß\nSince both choices can be activated for an input starting with \"abc\",\nNimbleParsec guarantees it will return the error from one of them, but\nnot which.","ref":"NimbleParsec.html#choice/3-beware-overlapping-choices"},{"type":"function","title":"NimbleParsec.concat/2","doc":"Concatenates two combinators.","ref":"NimbleParsec.html#concat/2"},{"type":"function","title":"Examples - NimbleParsec.concat/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :digit_upper_lower_plus,\n                concat(\n                  concat(ascii_char([?0..?9]), ascii_char([?A..?Z])),\n                  concat(ascii_char([?a..?z]), ascii_char([?+..?+]))\n                )\n    end\n\n    MyParser.digit_upper_lower_plus(\"1Az+\")\n    #=> {:ok, [?1, ?A, ?z, ?+], \"\", %{}, {1, 0}, 4}","ref":"NimbleParsec.html#concat/2-examples"},{"type":"function","title":"NimbleParsec.debug/2","doc":"Inspects the combinator state given to `to_debug` with the given `opts`.","ref":"NimbleParsec.html#debug/2"},{"type":"macro","title":"NimbleParsec.defcombinator/3","doc":"Defines a combinator with the given `name` and `opts`.\n\nIt is similar to `defparsec/3` except it does not define\nan entry-point parsing function, just the combinator function\nto be used with `parsec/2`.","ref":"NimbleParsec.html#defcombinator/3"},{"type":"macro","title":"NimbleParsec.defcombinatorp/3","doc":"Defines a combinator with the given `name` and `opts`.\n\nIt is similar to `defparsecp/3` except it does not define\nan entry-point parsing function, just the combinator function\nto be used with `parsec/2`.","ref":"NimbleParsec.html#defcombinatorp/3"},{"type":"macro","title":"NimbleParsec.defparsec/3","doc":"Defines a parser (and a combinator) with the given `name` and `opts`.\n\nThe parser is a function that receives two arguments, the binary\nto be parsed and a set of options. You can consult the documentation\nof the generated parser function for more information.\n\nThis function will also define a combinator that can be used as\n`parsec(name)` when building other parsers. See `parsec/2` for\nmore information on invoking compiled combinators.","ref":"NimbleParsec.html#defparsec/3"},{"type":"macro","title":"Beware! - NimbleParsec.defparsec/3","doc":"`defparsec/3` is executed during compilation. This means you can't\ninvoke a function defined in the same module. The following will error\nbecause the `date` function has not yet been defined:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      def date do\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n      end\n\n      defparsec :date, date()\n    end\n\nThis can be solved in different ways. You may simply\ncompose a long parser using variables. For example:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      date =\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n\n      defparsec :date, date\n    end\n\nAlternatively, you may define a `Helpers` module with many\nconvenience combinators, and then invoke them in your parser\nmodule:\n\n    defmodule MyParser.Helpers do\n      import NimbleParsec\n\n      def date do\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n      end\n    end\n\n    defmodule MyParser do\n      import NimbleParsec\n      import MyParser.Helpers\n\n      defparsec :date, date()\n    end\n\nThe approach of using helper modules is the favorite way\nof composing parsers in `NimbleParsec`.","ref":"NimbleParsec.html#defparsec/3-beware"},{"type":"macro","title":"Options - NimbleParsec.defparsec/3","doc":"* `:inline` - when true, inlines clauses that work as redirection for\n    other clauses. Settings this may improve runtime performance at the\n    cost of increased compilation time and bytecode size\n\n  * `:debug` - when true, writes generated clauses to `:stderr` for debugging\n\n  * `:export_combinator` - make the underlying combinator function public\n    so it can be used as part of `parsec/1` from other modules\n\n  * `:export_metadata` - export metadata necessary to use this parser\n    combinator to generate inputs","ref":"NimbleParsec.html#defparsec/3-options"},{"type":"macro","title":"NimbleParsec.defparsecp/3","doc":"Defines a private parser (and a combinator) with the given `name` and `opts`.\n\nThe same as `defparsec/3` but the parsing function is private.","ref":"NimbleParsec.html#defparsecp/3"},{"type":"function","title":"NimbleParsec.duplicate/3","doc":"Duplicates the combinator `to_duplicate` `n` times.","ref":"NimbleParsec.html#duplicate/3"},{"type":"function","title":"NimbleParsec.empty/0","doc":"Returns an empty combinator.\n\nAn empty combinator cannot be compiled on its own.","ref":"NimbleParsec.html#empty/0"},{"type":"function","title":"NimbleParsec.eos/1","doc":"Defines an end of string combinator.\n\nThe end of string does not produce a token and can be parsed multiple times.\nThis function is useful to avoid having to check for an empty remainder after\na successful parse.","ref":"NimbleParsec.html#eos/1"},{"type":"function","title":"Examples - NimbleParsec.eos/1","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :letter_pairs, utf8_string([], 2) |> repeat() |> eos()\n    end\n\n    MyParser.letter_pairs(\"hi\")\n    #=> {:ok, [\"hi\"], \"\", %{}, {1, 0}, 2}\n\n    MyParser.letter_pairs(\"hello\")\n    #=> {:error, \"expected end of string\", \"o\", %{}, {1, 0}, 4}","ref":"NimbleParsec.html#eos/1-examples"},{"type":"function","title":"NimbleParsec.eventually/2","doc":"Marks the given combinator should appear eventually.\n\nAny other data before the combinator appears is discarded.\nIf the combinator never appears, then it is an error.\n\n**Note:** this can be potentially a very expensive operation\nas it executes the given combinator byte by byte until finding\nan eventual match or ultimately failing. For example, if you\nare looking for an integer, it is preferrable to discard\neverything that is not an integer\n\n    ignore(ascii_string([not: ?0..?9]))\n\nrather than eventually look for an integer\n\n    eventually(ascii_string([?0..?9]))","ref":"NimbleParsec.html#eventually/2"},{"type":"function","title":"Examples - NimbleParsec.eventually/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      hour = integer(min: 1, max: 2)\n      defparsec :extract_hour, eventually(hour)\n    end\n\n    MyParser.extract_hour(\"let's meet at 12?\")\n    #=> {:ok, [12], \"?\", %{}, {1, 0}, 16}","ref":"NimbleParsec.html#eventually/2-examples"},{"type":"function","title":"NimbleParsec.generate/1","doc":"Generate a random binary from the given parsec.\n\nLet's see an example:\n\n    import NimbleParsec\n    generate(choice([string(\"foo\"), string(\"bar\")]))\n\nThe command above will return either \"foo\" or \"bar\". `generate/1`\nis often used with pre-defined parsecs. In this case, the\n`:export_metadata` flag must be set:\n\n    defmodule SomeModule do\n      import NimbleParsec\n      defparsec :parse,\n                choice([string(\"foo\"), string(\"bar\")]),\n                export_metadata: true\n    end\n\n    # Reference the parsec and generate from it\n    NimbleParsec.parsec({SomeModule, :parse})\n    |> NimbleParsec.generate()\n    |> IO.puts()\n\n`generate/1` can often run forever for recursive algorithms.\nRead the notes below and make use of the `gen_weight` and `gen_times`\noption to certain parsecs to control the recursion depth.","ref":"NimbleParsec.html#generate/1"},{"type":"function","title":"Notes - NimbleParsec.generate/1","doc":"Overall, there is no guarantee over the generated output, except\nthat it will generate a binary that is parseable by the parsec\nitself, but even this guarantee may be broken by parsers that have\ncustom validations. Keep in mind the following:\n\n  * `generate/1` is not compatible with NimbleParsec's dumped via\n    `mix nimble_parsec.compile`;\n\n  * `parsec/2` requires the referenced parsec to set `export_metadata: true`\n    on its definition;\n\n  * `choice/2` will be generated evenly. You can pass `:gen_weights`\n    as a list of positive integer weights to balance your choices.\n    This is particularly important for recursive algorithms;\n\n  * `repeat/2` and `repeat_while/3` will repeat between 0 and 3 times unless\n    a `:gen_times` option is given to these operations. `times/3` without a `:max`\n    will also additionally repeat between 0 and 3 times unless `:gen_times` is given.\n    The `:gen_times` option can either be an integer as the number of times to\n    repeat or a range where a random value in the range will be picked;\n\n  * `eventually/2` always generates the eventually parsec immediately;\n\n  * `lookahead/2` and `lookahead_not/2` are simply discarded;\n\n  * Validations done in any of the traverse definitions are not taken into account\n    by the generator. Therefore, if a parsec does validations, the generator may\n    generate binaries invalid to said parsec;","ref":"NimbleParsec.html#generate/1-notes"},{"type":"function","title":"NimbleParsec.ignore/2","doc":"Ignores the output of combinator given in `to_ignore`.","ref":"NimbleParsec.html#ignore/2"},{"type":"function","title":"Examples - NimbleParsec.ignore/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :ignorable, string(\"T\") |> ignore() |> integer(2)\n    end\n\n    MyParser.ignorable(\"T12\")\n    #=> {:ok, [12], \"\", %{}, {1, 0}, 2}","ref":"NimbleParsec.html#ignore/2-examples"},{"type":"function","title":"NimbleParsec.integer/2","doc":"Defines an integer combinator with of exact length or `min` and `max`\nlength.\n\nIf you want an integer of unknown size, use `integer(min: 1)`.\n\nThis combinator does not parse the sign and is always on base 10.","ref":"NimbleParsec.html#integer/2"},{"type":"function","title":"Examples - NimbleParsec.integer/2","doc":"With exact length:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :two_digits_integer, integer(2)\n    end\n\n    MyParser.two_digits_integer(\"123\")\n    #=> {:ok, [12], \"3\", %{}, {1, 0}, 2}\n\n    MyParser.two_digits_integer(\"1a3\")\n    #=> {:error, \"expected ASCII character in the range '0' to '9', followed by ASCII character in the range '0' to '9'\", \"1a3\", %{}, {1, 0}, 0}\n\nWith min and max:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :two_digits_integer, integer(min: 2, max: 4)\n    end\n\n    MyParser.two_digits_integer(\"123\")\n    #=> {:ok, [123], \"\", %{}, {1, 0}, 2}\n\n    MyParser.two_digits_integer(\"1a3\")\n    #=> {:error, \"expected ASCII character in the range '0' to '9', followed by ASCII character in the range '0' to '9'\", \"1a3\", %{}, {1, 0}, 0}\n\nIf the size of the integer has a min and max close to each other, such as\nfrom 2 to 4 or from 1 to 2, using choice may emit more efficient code:\n\n    choice([integer(4), integer(3), integer(2)])\n\nNote you should start from bigger to smaller.","ref":"NimbleParsec.html#integer/2-examples"},{"type":"function","title":"NimbleParsec.label/3","doc":"Adds a label to the combinator to be used in error reports.","ref":"NimbleParsec.html#label/3"},{"type":"function","title":"Examples - NimbleParsec.label/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :digit_and_lowercase,\n                empty()\n                |> ascii_char([?0..?9])\n                |> ascii_char([?a..?z])\n                |> label(\"digit followed by lowercase letter\")\n    end\n\n    MyParser.digit_and_lowercase(\"1a\")\n    #=> {:ok, [?1, ?a], \"\", %{}, {1, 0}, 2}\n\n    MyParser.digit_and_lowercase(\"a1\")\n    #=> {:error, \"expected a digit followed by lowercase letter\", \"a1\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#label/3-examples"},{"type":"function","title":"NimbleParsec.line/2","doc":"Puts the result of the given combinator as the first element\nof a tuple with the `line` as second element.\n\n`line` is a tuple where the first element is the current line\nand the second element is the byte offset immediately after\nthe newline.","ref":"NimbleParsec.html#line/2"},{"type":"function","title":"NimbleParsec.lookahead/2","doc":"Checks if a combinator is ahead.\n\nIf it succeeds, it continues as usual, otherwise it aborts the\nclosest `choice/2`, `repeat/2`, etc. If there is no closest\noperation to abort, then it errors.\n\nNote a lookahead never changes the accumulated output nor the\ncontext.","ref":"NimbleParsec.html#lookahead/2"},{"type":"function","title":"Examples - NimbleParsec.lookahead/2","doc":"For example, imagine you want to parse a language that has the\nkeywords \"if\" and \"while\" and identifiers made of any letters or\nnumber, where keywords and identifiers can be separated by a\nsingle white space:\n\n    defmodule IfWhileLang do\n      import NimbleParsec\n\n      keyword =\n        choice([\n          string(\"if\") |> replace(:if),\n          string(\"while\") |> replace(:while)\n        ])\n\n      identifier =\n        ascii_string([?a..?z, ?A..?Z, ?0..?9], min: 1)\n\n      defparsec :expr, repeat(choice([keyword, identifier]) |> optional(string(\" \")))\n    end\n\nThe issue with the implementation above is that the following\nwill parse:\n\n    IfWhileLang.expr(\"iffy\")\n    {:ok, [:if, \"fy\"], \"\", %{}, {1, 0}, 4}\n\nHowever, \"iffy\" should be treated as a full identifier. We could\nsolve this by inverting the order of `keyword` and `identifier`\nin `:expr` but that means \"if\" itself will be considered an identifier\nand not a keyword. To solve this, we need lookaheads.\n\nOne option is to check that after the keyword we either have an\nempty string OR the end of the string:\n\n    keyword =\n      choice([\n        string(\"if\") |> replace(:if),\n        string(\"while\") |> replace(:while)\n      ])\n      |> lookahead(choice([string(\" \"), eos()]))\n\nHowever, in this case, a negative lookahead may be clearer,\nand we can assert that we don't have any identifier character after\nthe keyword:\n\n    keyword =\n      choice([\n        string(\"if\") |> replace(:if),\n        string(\"while\") |> replace(:while)\n      ])\n      |> lookahead_not(ascii_char([?a..?z, ?A..?Z, ?0..?9]))\n\nNow we get the desired result back:\n\n    IfWhileLang.expr(\"iffy\")\n    #=> {:ok, [\"iffy\"], \"\", %{}, {1, 0}, 4}\n\n    IfWhileLang.expr(\"if fy\")\n    #=> {:ok, [:if, \" \", \"fy\"], \"\", %{}, {1, 0}, 5}","ref":"NimbleParsec.html#lookahead/2-examples"},{"type":"function","title":"NimbleParsec.lookahead_not/2","doc":"Checks if a combinator is not ahead.\n\nIf it succeeds, it aborts the closest `choice/2`, `repeat/2`, etc.\nOtherwise it continues as usual. If there is no closest operation\nto abort, then it errors.\n\nNote a lookahead never changes the accumulated output nor the\ncontext.\n\nFor an example, see `lookahead/2`.","ref":"NimbleParsec.html#lookahead_not/2"},{"type":"function","title":"NimbleParsec.map/3","doc":"Maps over the combinator results with the remote or local function in `call`.\n\n`call` is either a `{module, function, args}` representing\na remote call, a `{function, args}` representing a local call\nor an atom `function` representing `{function, []}`.\n\nEach parser result will be invoked individually for the `call`.\nEach result be prepended to the given `args`. The `args` will\nbe injected at the compile site and therefore must be escapable\nvia `Macro.escape/1`.\n\nSee `post_traverse/3` for a low level version of this function.","ref":"NimbleParsec.html#map/3"},{"type":"function","title":"Examples - NimbleParsec.map/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :letters_to_string_chars,\n                ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> map({Integer, :to_string, []})\n    end\n\n    MyParser.letters_to_string_chars(\"abc\")\n    #=> {:ok, [\"97\", \"98\", \"99\"], \"\", %{}, {1, 0}, 3}","ref":"NimbleParsec.html#map/3-examples"},{"type":"function","title":"NimbleParsec.optional/2","doc":"Marks the given combinator as `optional`.\n\nIt is equivalent to `choice([optional, empty()])`.","ref":"NimbleParsec.html#optional/2"},{"type":"function","title":"NimbleParsec.parsec/2","doc":"Invokes an already compiled combinator with name `name` in the\nsame module.\n\nEvery parser defined via `defparsec/3` or `defparsecp/3` can be\nused as combinators. However, the `defparsec/3` and `defparsecp/3`\nfunctions also define an entry-point parsing function, as implied\nby their names. If you want to define a combinator with the sole\npurpose of using it in combinator, use `defcombinatorp/3` instead.","ref":"NimbleParsec.html#parsec/2"},{"type":"function","title":"Use cases - NimbleParsec.parsec/2","doc":"`parsec/2` is useful to implement recursive definitions.\n\nNote while `parsec/2` can be used to compose smaller combinators,\nthe preferred mechanism for doing composition is via regular functions\nand not via `parsec/2`. Let's see a practical example. Imagine\nthat you have this module:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      date =\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n\n      time =\n        integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> optional(string(\"Z\"))\n\n      defparsec :datetime, date |> ignore(string(\"T\")) |> concat(time), debug: true\n    end\n\nNow imagine that you want to break `date` and `time` apart\ninto helper functions, as you use them in other occasions.\nGenerally speaking, you should **NOT** do this:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      defcombinatorp :date,\n                     integer(4)\n                     |> ignore(string(\"-\"))\n                     |> integer(2)\n                     |> ignore(string(\"-\"))\n                     |> integer(2)\n\n      defcombinatorp :time,\n                     integer(2)\n                     |> ignore(string(\":\"))\n                     |> integer(2)\n                     |> ignore(string(\":\"))\n                     |> integer(2)\n                     |> optional(string(\"Z\"))\n\n      defparsec :datetime,\n                parsec(:date) |> ignore(string(\"T\")) |> concat(parsec(:time))\n    end\n\nThe reason why the above is not recommended is because each\n`parsec/2` combinator ends-up adding a stacktrace entry during\nparsing, which affects the ability of `NimbleParsec` to optimize\ncode. If the goal is to compose combinators, you can do so\nwith modules and functions:\n\n    defmodule MyParser.Helpers do\n      import NimbleParsec\n\n      def date do\n        integer(4)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n        |> ignore(string(\"-\"))\n        |> integer(2)\n      end\n\n      def time do\n        integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> ignore(string(\":\"))\n        |> integer(2)\n        |> optional(string(\"Z\"))\n      end\n    end\n\n    defmodule MyParser do\n      import NimbleParsec\n      import MyParser.Helpers\n\n      defparsec :datetime,\n                date() |> ignore(string(\"T\")) |> concat(time())\n    end\n\nThe implementation above will be able to compile to the most\nefficient format as possible without forcing new stacktrace\nentries.\n\nThe only situation where you should use `parsec/2` for composition\nis when a large parser is used over and over again in a way\ncompilation times are high. In this sense, you can use `parsec/2`\nto improve compilation time at the cost of runtime performance.\nBy using `parsec/2`, the tree size built at compile time will be\nreduced although runtime performance is degraded as `parsec`\nintroduces a stacktrace entry.","ref":"NimbleParsec.html#parsec/2-use-cases"},{"type":"function","title":"Remote combinators - NimbleParsec.parsec/2","doc":"You can also reference combinators in other modules by passing\na tuple with the module name and a function to `parsec/2` as follows:\n\n    defmodule RemoteCombinatorModule do\n      defcombinator :upcase_unicode, utf8_char([...long, list, of, unicode, chars...])\n    end\n\n    defmodule LocalModule do\n      # Parsec that depends on `:upcase_A`\n      defparsec :parsec_name,\n                ...\n                |> ascii_char([?a..?Z])\n                |> parsec({RemoteCombinatorModule, :upcase_unicode})\n    end\n\nRemote combinators are useful when breaking the compilation of\nlarge modules apart in order to use Elixir's ability to compile\nmodules in parallel.","ref":"NimbleParsec.html#parsec/2-remote-combinators"},{"type":"function","title":"Examples - NimbleParsec.parsec/2","doc":"A good example of using `parsec` is with recursive parsers.\nA limited but recursive XML parser could be written as follows:\n\n    defmodule SimpleXML do\n      import NimbleParsec\n\n      tag = ascii_string([?a..?z, ?A..?Z], min: 1)\n      text = ascii_string([not: ?  concat(tag)\n        |> ignore(string(\">\"))\n\n      closing_tag =\n        ignore(string(\"  concat(tag)\n        |> ignore(string(\">\"))\n\n      defparsec :xml,\n                opening_tag\n                |> repeat(lookahead_not(string(\"  choice([parsec(:xml), text]))\n                |> concat(closing_tag)\n                |> wrap()\n    end\n\n    SimpleXML.xml(\" bar \")\n    #=> {:ok, [[\"foo\", \"bar\", \"foo\"]], \"\", %{}, {1, 0}, 14}\n\nIn the example above, `defparsec/3` has defined the entry-point\nparsing function as well as a combinator which we have invoked\nwith `parsec(:xml)`.\n\nIn many cases, however, you want to define recursive combinators\nwithout the entry-point parsing function. We can do this by\nreplacing `defparsec/3` by `defcombinatorp`:\n\n    defcombinatorp :xml,\n                   opening_tag\n                   |> repeat(lookahead_not(string(\"  choice([parsec(:xml), text]))\n                   |> concat(closing_tag)\n                   |> wrap()\n\nWhen using `defcombinatorp`, you can no longer invoke\n`SimpleXML.xml(xml)` as there is no associated parsing function.\nYou can only access the combinator above via `parsec/2`.","ref":"NimbleParsec.html#parsec/2-examples"},{"type":"function","title":"NimbleParsec.post_traverse/3","doc":"Traverses the combinator results with the remote or local function `call`.\n\n`call` is either a `{module, function, args}` representing\na remote call, a `{function, args}` representing a local call\nor an atom `function` representing `{function, []}`.\n\nThe function given in `call` will receive 5 additional arguments.\nThe rest of the parsed binary, the parser results to be post_traversed,\nthe parser context, the current line and the current offset will\nbe prepended to the given `args`. The `args` will be injected at\nthe compile site and therefore must be escapable via `Macro.escape/1`.\n\nThe line and offset will represent the location after the combinators.\nTo retrieve the position before the combinators, use `pre_traverse/3`.\n\nThe `call` must return a tuple `{rest, acc, context}` with list of\nresults to be added to the accumulator as first argument and a context\nas second argument. It may also return `{:error, reason}` to stop\nprocessing. Notice the received results are in reverse order and\nmust be returned in reverse order too.\n\nThe number of elements returned does not need to be\nthe same as the number of elements given.\n\nThis is a low-level function for changing the parsed result.\nOn top of this function, other functions are built, such as\n`map/3` if you want to map over each individual element and\nnot worry about ordering, `reduce/3` to reduce all elements\ninto a single one, `replace/3` if you want to replace the\nparsed result by a single value and `ignore/2` if you want to\nignore the parsed result.","ref":"NimbleParsec.html#post_traverse/3"},{"type":"function","title":"Examples - NimbleParsec.post_traverse/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :letters_to_chars,\n                ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> post_traverse({:join_and_wrap, [\"-\"]})\n\n      defp join_and_wrap(rest, args, context, _line, _offset, joiner) do\n        {rest, args |> Enum.join(joiner) |> List.wrap(), context}\n      end\n    end\n\n    MyParser.letters_to_chars(\"abc\")\n    #=> {:ok, [\"99-98-97\"], \"\", %{}, {1, 0}, 3}","ref":"NimbleParsec.html#post_traverse/3-examples"},{"type":"function","title":"NimbleParsec.pre_traverse/3","doc":"The same as `post_traverse/3` but receives the line and offset\nfrom before the wrapped combinators.\n\n`post_traverse/3` should be preferred as it keeps less stack\ninformation. Use `pre_traverse/3` only if you have to access\nthe line and offset from before the given combinators.","ref":"NimbleParsec.html#pre_traverse/3"},{"type":"function","title":"NimbleParsec.quoted_post_traverse/3","doc":"Invokes `call` to emit the AST that post traverses the `to_post_traverse`\ncombinator results.\n\nThis is similar to `post_traverse/3`. In `post_traverse/3`, `call` is\ninvoked to process the combinator results. In here, it is invoked to\nemit AST that in its turn will process the combinator results.\nThe invoked function must return the same types as `post_traverse/3`.\n\n`call` is a `{module, function, args}` and it will receive 5\nadditional arguments. The AST representation of the rest of the\nparsed binary, the parser results, context, line and offset will\nbe prepended to `args`. `call` is invoked at compile time and is\nuseful in combinators that avoid injecting runtime dependencies.\n\nThe line and offset will represent the location after the combinators.\nTo retrieve the position before the combinators, use `quoted_pre_traverse/3`.\n\nThis function must be used only when you want to emit code that\nhas no runtime dependencies in other modules. In most cases,\nusing `post_traverse/3` is better, since it doesn't work on ASTs\nand instead works at runtime.","ref":"NimbleParsec.html#quoted_post_traverse/3"},{"type":"function","title":"NimbleParsec.quoted_pre_traverse/3","doc":"The same as `quoted_post_traverse/3` but receives the line and offset\nfrom before the wrapped combinators.\n\n`quoted_post_traverse/3` should be preferred as it keeps less stack\ninformation. Use `quoted_pre_traverse/3` only if you have to access\nthe line and offset from before the given combinators.","ref":"NimbleParsec.html#quoted_pre_traverse/3"},{"type":"function","title":"NimbleParsec.quoted_repeat_while/4","doc":"Invokes `while` to emit the AST that will repeat `to_repeat`\nwhile the AST code returns `{:cont, context}`.\n\nIn case repetition should stop, `while` must return `{:halt, context}`.\n\n`while` is a `{module, function, args}` and it will receive 4\nadditional arguments. The AST representations of the binary to be\nparsed, context, line and offset will be prepended to `args`. `while`\nis invoked at compile time and is useful in combinators that avoid\ninjecting runtime dependencies.","ref":"NimbleParsec.html#quoted_repeat_while/4"},{"type":"function","title":"NimbleParsec.reduce/3","doc":"Reduces over the combinator results with the remote or local function in `call`.\n\n`call` is either a `{module, function, args}` representing\na remote call, a `{function, args}` representing a local call\nor an atom `function` representing `{function, []}`.\n\nThe parser results to be reduced will be prepended to the\ngiven `args`. The `args` will be injected at the compile site\nand therefore must be escapable via `Macro.escape/1`.\n\nSee `post_traverse/3` for a low level version of this function.","ref":"NimbleParsec.html#reduce/3"},{"type":"function","title":"Examples - NimbleParsec.reduce/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :letters_to_reduced_chars,\n                ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> ascii_char([?a..?z])\n                |> reduce({Enum, :join, [\"-\"]})\n    end\n\n    MyParser.letters_to_reduced_chars(\"abc\")\n    #=> {:ok, [\"97-98-99\"], \"\", %{}, {1, 0}, 3}","ref":"NimbleParsec.html#reduce/3-examples"},{"type":"function","title":"NimbleParsec.repeat/3","doc":"Allow the combinator given on `to_repeat` to appear zero or more times.\n\nBeware! Since `repeat/2` allows zero entries, it cannot be used inside\n`choice/2`, because it will always succeed and may lead to unused function\nwarnings since any further choice won't ever be attempted. For example,\nbecause `repeat/2` always succeeds, the `string/2` combinator below it\nwon't ever run:\n\n    choice([\n      repeat(ascii_char([?a..?z])),\n      string(\"OK\")\n    ])\n\nInstead of `repeat/2`, you may want to use `times/3` with the flags `:min`\nand `:max`.\n\nAlso beware! If you attempt to repeat a combinator that can match nothing,\nlike `optional/2`, `repeat/2` will not terminate. For example, consider\nthis combinator:\n\n     repeat(optional(utf8_char([?a])))\n\nThis combinator will never terminate because `repeat/2` chooses the empty\noption of `optional/2` every time. Since the goal of the parser above is\nto parse 0 or more `?a` characters, it can be represented by\n`repeat(utf8_char([?a]))`, because `repeat/2` allows 0 or more matches.","ref":"NimbleParsec.html#repeat/3"},{"type":"function","title":"Examples - NimbleParsec.repeat/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :repeat_lower, repeat(ascii_char([?a..?z]))\n    end\n\n    MyParser.repeat_lower(\"abcd\")\n    #=> {:ok, [?a, ?b, ?c, ?d], \"\", %{}, {1, 0}, 4}\n\n    MyParser.repeat_lower(\"1234\")\n    #=> {:ok, [], \"1234\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#repeat/3-examples"},{"type":"function","title":"NimbleParsec.repeat_while/4","doc":"Repeats while the given remote or local function `while` returns\n`{:cont, context}`.\n\nIf the combinator `to_repeat` stops matching, then the whole repeat\nloop stops successfully, hence it is important to assert the terminated\nvalue after repeating.\n\nIn case repetition should stop, `while` must return `{:halt, context}`.\n\n`while` is either a `{module, function, args}` representing\na remote call, a `{function, args}` representing a local call\nor an atom `function` representing `{function, []}`.\n\nThe function given in `while` will receive 4 additional arguments.\nThe `rest` of the binary to be parsed, the parser context, the\ncurrent line and the current offset will be prepended to the\ngiven `args`. The `args` will be injected at the compile site\nand therefore must be escapable via `Macro.escape/1`.","ref":"NimbleParsec.html#repeat_while/4"},{"type":"function","title":"Examples - NimbleParsec.repeat_while/4","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :string_with_quotes,\n                ascii_char([?\"])\n                |> repeat_while(\n                  choice([\n                    ~S(\\\") |> string() |> replace(?\"),\n                    utf8_char([])\n                  ]),\n                  {:not_quote, []}\n                )\n                |> ascii_char([?\"])\n                |> reduce({List, :to_string, []})\n\n      defp not_quote( >, context, _, _), do: {:halt, context}\n      defp not_quote(_, context, _, _), do: {:cont, context}\n    end\n\n    MyParser.string_with_quotes(~S(\"string with quotes \\\" inside\"))\n    {:ok, [\"\\\"string with quotes \\\" inside\\\"\"], \"\", %{}, {1, 0}, 30}\n\nNote you can use `lookahead/2` and `lookahead_not/2` with\n`repeat/2` (instead of `repeat_while/3`) to write a combinator\nthat repeats while a combinator matches (or does not match).\nFor example, the same combinator above could be written as:\n\n    defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :string_with_quotes,\n                ascii_char([?\"])\n                |> repeat(\n                  lookahead_not(ascii_char([?\"]))\n                  |> choice([\n                    ~S(\\\") |> string() |> replace(?\"),\n                    utf8_char([])\n                  ])\n                )\n                |> ascii_char([?\"])\n                |> reduce({List, :to_string, []})\n    end\n\n    MyParser.string_with_quotes(~S(\"string with quotes \\\" inside\"))\n    {:ok, [\"\\\"string with quotes \\\" inside\\\"\"], \"\", %{}, {1, 0}, 30}\n\nHowever, `repeat_while` is still useful when the condition to\nrepeat comes from the context passed around.","ref":"NimbleParsec.html#repeat_while/4-examples"},{"type":"function","title":"NimbleParsec.replace/3","doc":"Replaces the output of combinator given in `to_replace` by a single value.\n\nThe `value` will be injected at the compile site\nand therefore must be escapable via `Macro.escape/1`.","ref":"NimbleParsec.html#replace/3"},{"type":"function","title":"Examples - NimbleParsec.replace/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :replaceable, string(\"T\") |> replace(\"OTHER\") |> integer(2, 2)\n    end\n\n    MyParser.replaceable(\"T12\")\n    #=> {:ok, [\"OTHER\", 12], \"\", %{}, {1, 0}, 2}","ref":"NimbleParsec.html#replace/3-examples"},{"type":"function","title":"NimbleParsec.string/2","doc":"Defines a string binary value.","ref":"NimbleParsec.html#string/2"},{"type":"function","title":"Examples - NimbleParsec.string/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :string_t, string(\"T\")\n    end\n\n    MyParser.string_t(\"T\")\n    #=> {:ok, [\"T\"], \"\", %{}, {1, 0}, 1}\n\n    MyParser.string_t(\"not T\")\n    #=> {:error, \"expected a string \\\"T\\\"\", \"not T\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#string/2-examples"},{"type":"function","title":"NimbleParsec.tag/3","doc":"Tags the result of the given combinator in `to_tag` in a tuple with\n`tag` as first element.","ref":"NimbleParsec.html#tag/3"},{"type":"function","title":"Examples - NimbleParsec.tag/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :integer, integer(min: 1) |> tag(:integer)\n    end\n\n    MyParser.integer(\"1234\")\n    #=> {:ok, [integer: [1234]], \"\", %{}, {1, 0}, 4}\n\nNotice, however, that the integer result is wrapped in a list, because\nthe parser is expected to emit multiple tokens. When you are sure that\nonly a single token is emitted, you should use `unwrap_and_tag/3`.","ref":"NimbleParsec.html#tag/3-examples"},{"type":"function","title":"NimbleParsec.times/3","doc":"Allow the combinator given on `to_repeat` to appear at least, at most\nor exactly a given amount of times.","ref":"NimbleParsec.html#times/3"},{"type":"function","title":"Examples - NimbleParsec.times/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :minimum_lower, times(ascii_char([?a..?z]), min: 2)\n    end\n\n    MyParser.minimum_lower(\"abcd\")\n    #=> {:ok, [?a, ?b, ?c, ?d], \"\", %{}, {1, 0}, 4}\n\n    MyParser.minimum_lower(\"ab12\")\n    #=> {:ok, [?a, ?b], \"12\", %{}, {1, 0}, 2}\n\n    MyParser.minimum_lower(\"a123\")\n    #=> {:ok, [], \"a123\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#times/3-examples"},{"type":"function","title":"NimbleParsec.unwrap_and_tag/3","doc":"Unwraps and tags the result of the given combinator in `to_tag` in a tuple with\n`tag` as first element.","ref":"NimbleParsec.html#unwrap_and_tag/3"},{"type":"function","title":"Examples - NimbleParsec.unwrap_and_tag/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :integer, integer(min: 1) |> unwrap_and_tag(:integer)\n    end\n\n    MyParser.integer(\"1234\")\n    #=> {:ok, [integer: 1234], \"\", %{}, {1, 0}, 4}\n\n\nIn case the combinator emits greater than one token, an error will be raised.\nSee `tag/3` for more information.","ref":"NimbleParsec.html#unwrap_and_tag/3-examples"},{"type":"function","title":"NimbleParsec.utf8_char/2","doc":"Defines a single UTF-8 codepoint in the given ranges.\n\n`ranges` is a list containing one of:\n\n  * a `min..max` range expressing supported codepoints\n  * a `codepoint` integer expressing a supported codepoint\n  * `{:not, min..max}` expressing not supported codepoints\n  * `{:not, codepoint}` expressing a not supported codepoint","ref":"NimbleParsec.html#utf8_char/2"},{"type":"function","title":"Examples - NimbleParsec.utf8_char/2","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :digit_and_utf8,\n                empty()\n                |> utf8_char([?0..?9])\n                |> utf8_char([])\n    end\n\n    MyParser.digit_and_utf8(\"1é\")\n    #=> {:ok, [?1, ?é], \"\", %{}, {1, 0}, 2}\n\n    MyParser.digit_and_utf8(\"a1\")\n    #=> {:error, \"expected utf8 codepoint in the range '0' to '9', followed by utf8 codepoint\", \"a1\", %{}, {1, 0}, 0}","ref":"NimbleParsec.html#utf8_char/2-examples"},{"type":"function","title":"NimbleParsec.utf8_string/3","doc":"Defines an UTF8 string combinator with of exact length or `min` and `max`\ncodepoint length.\n\nThe `ranges` specify the allowed characters in the UTF8 string.\nSee `utf8_char/2` for more information.\n\nIf you want a string of unknown size, use `utf8_string(ranges, min: 1)`.\nIf you want a literal string, use `string/2`.\n\nNote that the combinator matches on codepoints, not graphemes. Therefore\nresults may vary depending on whether the input is in `nfc` or `nfd`\nnormalized form.","ref":"NimbleParsec.html#utf8_string/3"},{"type":"function","title":"Examples - NimbleParsec.utf8_string/3","doc":"defmodule MyParser do\n      import NimbleParsec\n\n      defparsec :two_letters, utf8_string([], 2)\n    end\n\n    MyParser.two_letters(\"áé\")\n    #=> {:ok, [\"áé\"], \"\", %{}, {1, 0}, 3}","ref":"NimbleParsec.html#utf8_string/3-examples"},{"type":"function","title":"NimbleParsec.wrap/2","doc":"Wraps the results of the given combinator in `to_wrap` in a list.","ref":"NimbleParsec.html#wrap/2"},{"type":"type","title":"NimbleParsec.bin_modifier/0","doc":"","ref":"NimbleParsec.html#t:bin_modifier/0"},{"type":"type","title":"NimbleParsec.call/0","doc":"","ref":"NimbleParsec.html#t:call/0"},{"type":"type","title":"NimbleParsec.exclusive_range/0","doc":"","ref":"NimbleParsec.html#t:exclusive_range/0"},{"type":"type","title":"NimbleParsec.fargs/0","doc":"","ref":"NimbleParsec.html#t:fargs/0"},{"type":"type","title":"NimbleParsec.gen_times/0","doc":"","ref":"NimbleParsec.html#t:gen_times/0"},{"type":"type","title":"NimbleParsec.gen_weights/0","doc":"","ref":"NimbleParsec.html#t:gen_weights/0"},{"type":"type","title":"NimbleParsec.inclusive_range/0","doc":"","ref":"NimbleParsec.html#t:inclusive_range/0"},{"type":"type","title":"NimbleParsec.mfargs/0","doc":"","ref":"NimbleParsec.html#t:mfargs/0"},{"type":"type","title":"NimbleParsec.min_and_max/0","doc":"","ref":"NimbleParsec.html#t:min_and_max/0"},{"type":"type","title":"NimbleParsec.opts/0","doc":"","ref":"NimbleParsec.html#t:opts/0"},{"type":"type","title":"NimbleParsec.range/0","doc":"","ref":"NimbleParsec.html#t:range/0"},{"type":"opaque","title":"NimbleParsec.t/0","doc":"","ref":"NimbleParsec.html#t:t/0"}],"content_type":"text/markdown"}